"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getOptionsFromGithub = void 0;
const BackportError_1 = require("../../../BackportError");
const git_1 = require("../../../git");
const logger_1 = require("../../../logger");
const remoteConfig_1 = require("../../../remoteConfig");
const apiRequestV4_1 = require("../apiRequestV4");
const throwOnInvalidAccessToken_1 = require("../throwOnInvalidAccessToken");
const query_1 = require("./query");
async function getOptionsFromGithub(options) {
    const { accessToken, githubApiBaseUrlV4, repoName, repoOwner, globalConfigFile, } = options;
    let data;
    try {
        const res = await (0, apiRequestV4_1.apiRequestV4)({
            githubApiBaseUrlV4,
            accessToken,
            query: query_1.query,
            variables: { repoOwner, repoName },
            fullResponse: true,
        });
        throwIfInsufficientPermissions(res);
        data = res.data.data;
    }
    catch (e) {
        if (!(e instanceof apiRequestV4_1.GithubV4Exception)) {
            throw e;
        }
        (0, throwOnInvalidAccessToken_1.throwOnInvalidAccessToken)({
            error: e,
            repoName,
            repoOwner,
            globalConfigFile,
        });
        data = (0, remoteConfig_1.swallowMissingConfigFileException)(e);
    }
    // it is not possible to have a branch named "backport"
    if (data.repository.illegalBackportBranch) {
        throw new BackportError_1.BackportError('You must delete the branch "backport" to continue. See https://github.com/sqren/backport/issues/155 for details');
    }
    const remoteConfig = await getRemoteConfigFileOptions(data, options.cwd, options.skipRemoteConfig);
    return {
        authenticatedUsername: data.viewer.login,
        sourceBranch: data.repository.defaultBranchRef.name,
        isRepoPrivate: data.repository.isPrivate,
        ...remoteConfig,
    };
}
exports.getOptionsFromGithub = getOptionsFromGithub;
async function getRemoteConfigFileOptions(res, cwd, skipRemoteConfig) {
    if (skipRemoteConfig) {
        logger_1.logger.info('Remote config: Skipping. `--skip-remote-config` specified via config file or cli');
        return;
    }
    const remoteConfig = res.repository.defaultBranchRef.target.remoteConfigHistory.edges?.[0]
        ?.remoteConfig;
    if (!remoteConfig) {
        logger_1.logger.info("Remote config: Skipping. Remote config doesn't exist");
        return;
    }
    if (cwd) {
        const [isLocalConfigModified, isLocalConfigUntracked, localCommitDate] = await Promise.all([
            (0, git_1.isLocalConfigFileModified)({ cwd }),
            (0, git_1.isLocalConfigFileUntracked)({ cwd }),
            (0, git_1.getLocalConfigFileCommitDate)({ cwd }),
        ]);
        if (isLocalConfigUntracked) {
            logger_1.logger.info('Remote config: Skipping. Local config is new');
            return;
        }
        if (isLocalConfigModified) {
            logger_1.logger.info('Remote config: Skipping. Local config is modified');
            return;
        }
        if (localCommitDate &&
            localCommitDate > Date.parse(remoteConfig.committedDate)) {
            logger_1.logger.info(`Remote config: Skipping. Local config is newer: ${new Date(localCommitDate).toISOString()} > ${remoteConfig.committedDate}`);
            return;
        }
    }
    logger_1.logger.info('Remote config: Parsing.');
    return (0, remoteConfig_1.parseRemoteConfig)(remoteConfig);
}
function throwIfInsufficientPermissions(res) {
    const accessScopesHeader = res.headers['x-oauth-scopes'];
    if (accessScopesHeader === undefined) {
        return;
    }
    const accessTokenScopes = accessScopesHeader
        .split(',')
        .map((scope) => scope.trim());
    const isRepoPrivate = res.data.data.repository.isPrivate;
    if (isRepoPrivate && !accessTokenScopes.includes('repo')) {
        throw new BackportError_1.BackportError(`You must grant the "repo" scope to your personal access token`);
    }
    if (!accessTokenScopes.includes('repo') &&
        !accessTokenScopes.includes('public_repo')) {
        throw new BackportError_1.BackportError(`You must grant the "repo" or "public_repo" scope to your personal access token`);
    }
}
//# sourceMappingURL=getOptionsFromGithub.js.map